#pragma once
#if defined(_WIN32)
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#define AGORA_CALL __cdecl
#if defined(AGORARTC_EXPORT)
#define AGORA_API extern "C" __declspec(dllexport)
#else
#define AGORA_API extern "C" __declspec(dllimport)
#endif
#elif defined(__APPLE__)
#define AGORA_API __attribute__((visibility("default"))) extern "C"
#define AGORA_CALL
#elif defined(__ANDROID__) || defined(__linux__)
#define AGORA_API extern "C" __attribute__((visibility("default")))
#define AGORA_CALL
#else
#define AGORA_API extern "C"
#define AGORA_CALL
#endif

////////////////////////////////////////////////////////
/** \addtogroup AG_MEDIA_PLAYER_STATE
 @{
 */
 ////////////////////////////////////////////////////////

 /**
  Current state of MediaPlayer.
  */
enum AG_MEDIA_PLAYER_STATE

{
	/** MediaPlayer is not initialized. */
	AG_MEDIA_PLAYER_STATE_UNINITIALIZED = 0,
	/** MediaPlayer is ready for play. */
	AG_MEDIA_PLAYER_STATE_READY = 1,
	/** MediaPlayer is playing video. */
	AG_MEDIA_PLAYER_STATE_PLAY = 2,
	/** The video is paused. */
	AG_MEDIA_PLAYER_STATE_PAUSE = 3,
	/** The video is stopped. */
	AG_MEDIA_PLAYER_STATE_STOP = 4,
	/** The video is buffering. */
	AG_MEDIA_PLAYER_STATE_BUFFERING = 5,
	/** The video playback is complete. */
	AG_MEDIA_PLAYER_STATE_COMPLETE = 6,

};

////////////////////////////////////////////////////////
/** @} */
////////////////////////////////////////////////////////

////////////////////////////////////////////////////////
/** \addtogroup AG_KIT_ERROR
 @{
 */
 ////////////////////////////////////////////////////////

 /**
  MediaPlayer playback error codes.
  */
enum AG_KIT_ERROR
{
	/** Used the unsupported argument. */
	AG_SOURCE_LOADED_ERROR_UNSUPPORTED_ARGUMENT = 1,
	/** The data is corrupted. */
	AG_SOURCE_LOADED_ERROR_CORRUPTED_DATA = 2,
	/** Used the unsupported format. */
	AG_SOURCE_LOADED_ERROR_UNSUPPORTED_FORMAT = 3,
	/** No available audio track. */
	AG_SOURCE_LOADED_ERROR_NO_AVAILABLE_TRACK = 4,
	/** No available network. */
	AG_PLAY_ERROR_NETWORK_UNAVAILABLE = 7,
};

////////////////////////////////////////////////////////
/** @} */
////////////////////////////////////////////////////////

/**
 The SDK uses the MediaPlayerKitEventHandler interface class to send callbacks
 to the application.

 In the callbacks, avoid time-consuming tasks or calling blocking APIs, such as
 the load method. Otherwise, the SDK may not work properly.
 */
class MediaPlayerKitEventHandler
{
public:
	/**
	 Reports the current state of MediaPlayer.

	 @param state Currents state of MediaPlayer, see #AG_MEDIA_PLAYER_STATE
	 */
	virtual void onPlayerStateChanged(AG_MEDIA_PLAYER_STATE state) {

	}
	/**
	 Reports the MediaPlayer playback error codes.

	 @param error The playback error codes, see #AG_KIT_ERROR
	 */
	virtual void onKitError(AG_KIT_ERROR error) {

	}
};

/**
 Reports the media information.

 In the callbacks, avoid time-consuming tasks or calling blocking APIs, such as
 the load method. Otherwise, the SDK may not work properly.
 */
class MediaInfoCallback {
public:
	/**
	 Reports the audio track information.

	 @param index The index of a current audio stream, representing the number
	 of the audio streams.
	 @param sample_rate The audio sample rate (Hz).
	 @param channels The number of channels (mono or dual).
	 @param duration The duration (ms) of the video.
	 */
	virtual void onAudioTrackInfoCallback(int index, int sample_rate, int channels, long duration) {

	}
	/**
	 Reports the video track information.

	 @param index The index of a current video stream, representing the number of
	 the video streams.
	 @param width The video width (pixel).
	 @param height The video height (pixel).
	 */
	virtual void onVideoTrackInfoCallback(int index, int width, int height) {

	}
};

/**
 MediaPlayerKit is the base interface class and provides the main methods
 invoked by your application.
 */
class MediaPlayerKit
{
public:
	/**
	 Sets the MediaPlayerKitEventHandler class.

	 @param handler MediaPlayerKitEventHandler
	 */
	virtual void setEventHandler(MediaPlayerKitEventHandler *handler) = 0;
	/**
	 Sets the MediaInfoCallback class.

	 @param callback MediaInfoCallback
	 */
	virtual void setMediaInfoCallback(MediaInfoCallback *callback) = 0;
	/**
	 Sets the playback video window and configures the playback video display
	 settings on the device.

	 The application calls this method to bind each video window of the
	 video and configures the video display settings.

	 - If you use MediaPlayer as a local media player, call this method after initialization.
	 - If you want to publish the playback video to remote users, call this method after
	 initialization and before joining the channel.

	 When you destroy a MediaPlayerKit object, the binding here is not valid.

	 @param hwnd Input parameter for displaying the video window.
	 */
	virtual void setVideoView(void *hwnd) = 0;
	/**
	 Loads a specified video into the memory.

	 @note Call this method after joining the channel when you want to publish
	 the video to remote users.

	 @param source The URL address or the local file path of a specified video.
	 @param auto_play Sets whether to play the video automatically after loading:
	 - true: Play the video automatically
	 - false: Does not play the video automatically
	 */
	virtual void load(char *source, bool auto_play) = 0;
	/**
	 Retrieves the duration (ms) of the current video.

	 @warning You can not retrieves the duration of the real-time video streams.

	 You can retrieve the duration of the current video only when you have received the AG_MEDIA_PLAYER_STATE_READY state code,
	 otherwise the duration is NULL.

	 @return
	 - &ge; 0:  The video duration, if the method call is successful
	 - < 0: Failure
	 */
	virtual long getDuration() = 0;
	/**
	 Plays a specified video.
	 */
	virtual void play() = 0;
	/**
	 Pauses a specified video.
	 */
	virtual void pause() = 0;
	/**
	 Resumes playing a specified video.
	 */
	virtual void resume() = 0;
	/**
	 Retrieves the playback position (ms) of the current video.

	 @warning You can not retrieves the duration of the real-time video streams.

	 @return
	 - &ge; 0: The video position, if the method call is successful
	 - < 0: Failure
	 */
	virtual long getCurrentPosition() = 0;
	/**
	 Sets the playback position (ms) of the current video.

	 This method sets the playback position (ms) of the current video to a
	 different starting position (the default starting position is the beginning).

	 @param msec The playback starting position (ms) of the current video.
	 */
	virtual void seekTo(long msec) = 0;
	/**
	 Adjusts the local playback volume.

	 @param volume The playback volume. The value ranges between 0 and 400.

	 @note
	 - The value of `volume` is not measured in decibels but in 1. Agora defines
	 original volume to 100.
	 - 400: (Maximum) Four times the original volume with signal clipping protection.
	 - 0: Mute
	 */
	virtual void adjustPlaybackSignalVolume(int volume) = 0;
	/**
	 Stops playing a specified video.
	 */
	virtual void stop() = 0;
	/**
	 Releases a specified loaded video from the memory.
	 */
	virtual void unload() = 0;
	/**
	 Destroys a MediaPlayerKit instance.
	 */
	virtual void destroy() = 0;
	/**
	 Publishes the playback audio to remote users.

	 This method publishes the playback audio to remote users in the channel.
	 */
	virtual void publishAudio() = 0;
	/**
	 Stops publishing the playback audio to remote users.

	 This method stops publishing the playback audio to remote users in the channel.
	 */
	virtual void unpublishAudio() = 0;
	/**
	 Publishes the playback video to remote users.

	 This method publishes the playback video to remote users in the channel.
	 */
	virtual void publishVideo() = 0;
	/**
	 Stops publishing the playback video to remote users.

	 This method stops publishing the playback video to remote users in the channel.
	 */
	virtual void unpublishVideo() = 0;
	/**
	 Adjusts the video volume for publishing to remote users.

	 @note Call this method after joining the channel when you want to publish
	 the video to remote users.

	 @param volume The playback volume. The value ranges between 0 and 400.

	 @note
	 - The value of `volume` is not measured in decibels but in 1. Agora defines
	 original volume to 100.
	 - 400: (Maximum) Four times the original volume with signal clipping protection.
	 - 0: Mute
	 */
	virtual void adjustPublishSignalVolume(int volume) = 0;
};

////////////////////////////////////////////////////////
/** \addtogroup createMediaPlayerKit
 @{
 */
 ////////////////////////////////////////////////////////

 /**
  Creates the MediaPlayerKit object and returns the pointer.

  @note You can use this method to create the engine when you use the MediaPlayer as a local media player.

  @return Pointer to the MediaPlayerKit object.
  */
AGORA_API MediaPlayerKit * AGORA_CALL createMediaPlayerKit();

////////////////////////////////////////////////////////
/** @} */
////////////////////////////////////////////////////////

////////////////////////////////////////////////////////
/** \addtogroup createMediaPlayerKitWithRtcEngine
 @{
 */
 ////////////////////////////////////////////////////////

 /**
  Creates the MediaPlayerKitWithRtcEngine object and returns the pointer.

  Creates the MediaPlayerKit engine with the AgoraRtcEngine and the audio
  sample rate.

  @note
  - You can use this method to create the engine when you want to publish the video played by MediaPlayer to remote users.
  - The parameter passed into AgoraRtcEngine (`engine`) cannot be NULL.

  @param engine AgoraRTCEngine
  @param sample_rate The audio sample rate (Hz)

  @return Pointer to the MediaPlayerKitWithRtcEngine object.
  */
AGORA_API MediaPlayerKit * AGORA_CALL createMediaPlayerKitWithRtcEngine(void * engine, int sample_rate);

////////////////////////////////////////////////////////
/** @} */
////////////////////////////////////////////////////////
